@@ -151,6 +151,8 @@ d_p_k <- function(p, k, log = FALSE) {
 #' @export
 weights_dfs_Sobolev <- function(p, K_max = 1e3, thre = 1e-3, type,
                                 Rothman_t = 1 / 3, Pycke_q = 0.5, Riesz_s = 1,
+                                Poisson_rho = 0.5, Softmax_kappa = 1,
+                                Stereo_a = 0, Sobolev_vk2 = c(0, 0, 1),
                                 log = FALSE, verbose = TRUE, Gauss = TRUE,
                                 N = 320, tol = 1e-6, force_positive = TRUE,
                                 x_tail = NULL) {
@@ -159,7 +161,7 @@ weights_dfs_Sobolev <- function(p, K_max = 1e3, thre = 1e-3, type,
   alpha <- 0.5 * p - 1
 
   # Sobolev weights and dfs
-  if (p == 2 & type %in% c("Watson", "Rothman", "Hermans_Rasson", "Pycke_q")) {
+  if (p == 2 && type %in% c("Watson", "Rothman", "Hermans_Rasson", "Pycke_q")) {
 
     if (type == "Watson") {
 
@@ -193,6 +195,9 @@ weights_dfs_Sobolev <- function(p, K_max = 1e3, thre = 1e-3, type,
 
     } else if (type == "Hermans_Rasson") {
 
+      # Halve K_max since we compute the odd/even coefficients separately
+      K_max <- K_max %/% 2
+
       # Sequence of indexes
       k <- 1:K_max
 
@@ -306,22 +311,14 @@ weights_dfs_Sobolev <- function(p, K_max = 1e3, thre = 1e-3, type,
       # Sequence of indexes
       k <- 1:K_max
 
-      # log(v_k^2)
+      # log(b_k)
       log_vk2 <- switch((p > 2) + 1, log(8) - log(pi) - log(4 * k^2 - 1),
                         (p - 3) * log(2) + log(2 * k + p - 2) +
-                          lgamma(k - 1 / 2) + lgamma(p / 2 - 1) +
+                          lgamma(k - 1 / 2) + lgamma(alpha) +
                           lgamma(p / 2) - log(pi) - lgamma(k + p - 1 / 2))
 
-      # Divide by 2 if p = 2 and (1 + k / alpha) if p > 2
-      if (p == 2) {
-
-        log_vk2 <- log_vk2 - log(2)
-
-      } else {
-
-        log_vk2 <- log_vk2 - log(1 + k / alpha)
-
-      }
+      # Switch from bk to vk2
+      log_vk2 <- bk_to_vk2(bk = log_vk2, p = p, log = TRUE)
 
       # log(d_{p, k})
       log_dk <- d_p_k(p = p, k = k, log = TRUE)
@@ -377,23 +374,15 @@ weights_dfs_Sobolev <- function(p, K_max = 1e3, thre = 1e-3, type,
 
         }
 
-        # log(v_k^2) = log(b_{s, k, p})
+        # log(b_k)
         log_vk2 <- log(tau / sqrt(pi)) + lgamma((p - 1 + Riesz_s) / 2) +
           lgamma(-Riesz_s / 2 + k) - lgamma(p - 1 + Riesz_s / 2 + k) -
           lgamma(-Riesz_s / 2)
 
       }
 
-      # Divide by 2 if p = 2 and (1 + k / alpha) if p > 2
-      if (p == 2) {
-
-        log_vk2 <- log_vk2 - log(2)
-
-      } else {
-
-        log_vk2 <- log_vk2 - log(1 + k / alpha)
-
-      }
+      # Switch from bk to vk2
+      log_vk2 <- bk_to_vk2(bk = log_vk2, p = p, log = TRUE)
 
       # log(d_{p, k})
       log_dk <- d_p_k(p = p, k = k, log = TRUE)
@@ -407,20 +396,12 @@ weights_dfs_Sobolev <- function(p, K_max = 1e3, thre = 1e-3, type,
       # Sequence of indexes
       k <- 1:K_max
 
-      # log(v_k^2)
+      # log(b_k)
       log_vk2 <- log(Gegen_coefs_Pn(k = k, p = p, type = "PCvM",
                                     Gauss = Gauss, N = N, tol = tol))
 
-      # Divide by 2 if p = 2 and (1 + k / alpha) if p > 2
-      if (p == 2) {
-
-        log_vk2 <- log_vk2 - log(2)
-
-      } else {
-
-        log_vk2 <- log_vk2 - log(1 + k / alpha)
-
-      }
+      # Switch from bk to vk2
+      log_vk2 <- bk_to_vk2(bk = log_vk2, p = p, log = TRUE)
 
       # log(d_{p, k})
       log_dk <- d_p_k(p = p, k = k, log = TRUE)
@@ -434,20 +415,12 @@ weights_dfs_Sobolev <- function(p, K_max = 1e3, thre = 1e-3, type,
       # Sequence of indexes
       k <- 1:K_max
 
-      # log(v_k^2)
+      # log(b_k)
       log_vk2 <- log(Gegen_coefs_Pn(k = k, p = p, type = "PAD",
                                     Gauss = Gauss, N = N, tol = tol))
 
-      # Divide by 2 if p = 2 and (1 + k / alpha) if p > 2
-      if (p == 2) {
-
-        log_vk2 <- log_vk2 - log(2)
-
-      } else {
-
-        log_vk2 <- log_vk2 - log(1 + k / alpha)
-
-      }
+      # Switch from bk to vk2
+      log_vk2 <- bk_to_vk2(bk = log_vk2, p = p, log = TRUE)
 
       # log(d_{p, k})
       log_dk <- d_p_k(p = p, k = k, log = TRUE)
@@ -461,21 +434,13 @@ weights_dfs_Sobolev <- function(p, K_max = 1e3, thre = 1e-3, type,
       # Sequence of indexes
       k <- 1:K_max
 
-      # log(v_k^2)
+      # log(b_k)
       log_vk2 <- log(Gegen_coefs_Pn(k = k, p = p, type = "PRt",
                                     Rothman_t = Rothman_t, Gauss = Gauss,
                                     N = N, tol = tol))
 
-      # Divide by 2 if p = 2 and (1 + k / alpha) if p > 2
-      if (p == 2) {
-
-        log_vk2 <- log_vk2 - log(2)
-
-      } else {
-
-        log_vk2 <- log_vk2 - log(1 + k / alpha)
-
-      }
+      # Switch from bk to vk2
+      log_vk2 <- bk_to_vk2(bk = log_vk2, p = p, log = TRUE)
 
       # log(d_{p, k})
       log_dk <- d_p_k(p = p, k = k, log = TRUE)
@@ -484,6 +449,102 @@ weights_dfs_Sobolev <- function(p, K_max = 1e3, thre = 1e-3, type,
       log_weights <- log_vk2
       log_dfs <- log_dk
 
+    } else if (type == "Poisson") {
+
+      # Sequence of indexes
+      k <- 1:K_max
+
+      # log(b_k)
+      if (p == 2) {
+
+        log_vk2 <- log(2 - (k == 0)) + k * log(Poisson_rho)
+
+      } else {
+
+        log_vk2 <- log(2 * k + p - 2) - log(p - 2) + k * log(Poisson_rho)
+
+      }
+
+      # Switch from bk to vk2
+      log_vk2 <- bk_to_vk2(bk = log_vk2, p = p, log = TRUE)
+
+      # log(d_{p, k})
+      log_dk <- d_p_k(p = p, k = k, log = TRUE)
+
+      # Log weights and dfs
+      log_weights <- log_vk2
+      log_dfs <- log_dk
+
+    } else if (type == "Softmax") {
+
+      # Sequence of indexes
+      k <- 1:K_max
+
+      # log(b_k)
+      if (p == 2) {
+
+        log_vk2 <- log(2 - (k == 0)) +
+          log(besselI(x = Softmax_kappa, nu = k, expon.scaled = TRUE))
+
+      } else {
+
+        log_vk2 <- alpha * log(2 / Softmax_kappa) + lgamma(alpha) +
+          log(k + alpha) + log(besselI(x = Softmax_kappa, nu = k + alpha,
+                                       expon.scaled = TRUE))
+
+      }
+
+      # Switch from bk to vk2
+      log_vk2 <- bk_to_vk2(bk = log_vk2, p = p, log = TRUE)
+
+      # log(d_{p, k})
+      log_dk <- d_p_k(p = p, k = k, log = TRUE)
+
+      # Log weights and dfs
+      log_weights <- log_vk2
+      log_dfs <- log_dk
+
+    } else if (type == "Stereo") {
+
+      # Halve K_max since we compute the odd/even coefficients separately
+      K_max <- K_max %/% 2
+
+      # Sequence of indexes
+      k <- 1:K_max
+
+      # log(v_{2 * k - 1}^2)
+      log_v2km12 <- log(2 * k - 1) + log(4 * (k - 1) + p) - log(2 * k + p - 3) +
+        log(1 - Stereo_a)
+      log_alpha_2km1 <- 2 * (lgamma(k - 0.5) + lgamma(alpha) -
+                               lgamma(k + alpha - 0.5)) - log(2 * pi)
+
+      # log(v_{2 * k}^2)
+      log_v2k2 <- log(4 * k + p - 2) + log(1 + Stereo_a)
+      log_alpha_2k <- 2 * (lgamma(k + 0.5) + lgamma(alpha) -
+                             lgamma(k + 0.5 * (p - 1))) - log(2 * pi)
+
+      # log(b_k)
+      log_vk2 <- c(rbind(log_alpha_2km1 + log_v2km12, log_alpha_2k + log_v2k2))
+
+      # Switch from bk to vk2
+      log_vk2 <- bk_to_vk2(bk = log_vk2, p = p, log = TRUE)
+
+      # log(d_{p, k})
+      log_dk <- d_p_k(p = p, k = 1:(2 * K_max), log = TRUE)
+
+      # Log weights and dfs
+      log_weights <- log_vk2
+      log_dfs <- log_dk
+
+    } else if (type == "Sobolev") {
+
+      # log(d_{p, k})
+      log_dk <- d_p_k(p = p, k = seq_along(Sobolev_vk2), log = TRUE)
+
+      # Log weights and dfs
+      log_weights <- log(Sobolev_vk2)
+      log_dfs <- log_dk
+
     } else {
 
       stop("Incompatible choice of p and type.")
@@ -533,12 +594,17 @@ weights_dfs_Sobolev <- function(p, K_max = 1e3, thre = 1e-3, type,
 #' @export
 d_Sobolev <- function(x, p, type, method = c("I", "SW", "HBE")[1], K_max = 1e3,
                       thre = 1e-3, Rothman_t = 1 / 3, Pycke_q = 0.5,
-                      Riesz_s = 1, ncps = 0, verbose = TRUE, N = 320,
-                      x_tail = NULL, ...) {
+                      Riesz_s = 1, Poisson_rho = 0.5, Softmax_kappa = 1,
+                      Stereo_a = 0, Sobolev_vk2 = c(0, 0, 1), ncps = 0,
+                      verbose = TRUE, N = 320, x_tail = NULL, ...) {
 
   weights_dfs <- weights_dfs_Sobolev(p = p, K_max = K_max, thre = thre,
                                      type = type, Rothman_t = Rothman_t,
                                      Pycke_q = Pycke_q, Riesz_s = Riesz_s,
+                                     Poisson_rho = Poisson_rho,
+                                     Softmax_kappa = Softmax_kappa,
+                                     Stereo_a = Stereo_a,
+                                     Sobolev_vk2 = Sobolev_vk2,
                                      verbose = verbose, Gauss = TRUE, N = N,
                                      x_tail = x_tail)
   d_wschisq(x = x, weights = weights_dfs$weights, dfs = weights_dfs$dfs,
@@ -551,14 +617,19 @@ d_Sobolev <- function(x, p, type, method = c("I", "SW", "HBE")[1], K_max = 1e3,
 #' @export
 p_Sobolev <- function(x, p, type, method = c("I", "SW", "HBE", "MC")[1],
                       K_max = 1e3, thre = 1e-3, Rothman_t = 1 / 3,
-                      Pycke_q = 0.5, Riesz_s = 1, ncps = 0, verbose = TRUE,
-                      N = 320, x_tail = NULL, ...) {
+                      Pycke_q = 0.5, Riesz_s = 1, Poisson_rho = 0.5,
+                      Softmax_kappa = 1, Stereo_a = 0, Sobolev_vk2 = c(0, 0, 1),
+                      ncps = 0, verbose = TRUE, N = 320, x_tail = NULL, ...) {
 
   weights_dfs <- weights_dfs_Sobolev(p = p, K_max = K_max, thre = thre,
                                      type = type, Rothman_t = Rothman_t,
                                      Pycke_q = Pycke_q, Riesz_s = Riesz_s,
-                                     verbose = verbose, Gauss = TRUE,
-                                     N = N, x_tail = x_tail)
+                                     Poisson_rho = Poisson_rho,
+                                     Softmax_kappa = Softmax_kappa,
+                                     Stereo_a = Stereo_a,
+                                     Sobolev_vk2 = Sobolev_vk2,
+                                     verbose = verbose, Gauss = TRUE, N = N,
+                                     x_tail = x_tail)
   p_wschisq(x = x, weights = weights_dfs$weights, dfs = weights_dfs$dfs,
             ncps = ncps, method = method, ...)
 
@@ -569,15 +640,106 @@ p_Sobolev <- function(x, p, type, method = c("I", "SW", "HBE", "MC")[1],
 #' @export
 q_Sobolev <- function(u, p, type, method = c("I", "SW", "HBE", "MC")[1],
                       K_max = 1e3, thre = 1e-3, Rothman_t = 1 / 3,
-                      Pycke_q = 0.5, Riesz_s = 1, ncps = 0, verbose = TRUE,
-                      N = 320, x_tail = NULL, ...) {
+                      Pycke_q = 0.5, Riesz_s = 1, Poisson_rho = 0.5,
+                      Softmax_kappa = 1, Stereo_a = 0, Sobolev_vk2 = c(0, 0, 1),
+                      ncps = 0, verbose = TRUE, N = 320, x_tail = NULL, ...) {
 
   weights_dfs <- weights_dfs_Sobolev(p = p, K_max = K_max, thre = thre,
                                      type = type, Rothman_t = Rothman_t,
                                      Pycke_q = Pycke_q, Riesz_s = Riesz_s,
+                                     Poisson_rho = Poisson_rho,
+                                     Softmax_kappa = Softmax_kappa,
+                                     Stereo_a = Stereo_a,
+                                     Sobolev_vk2 = Sobolev_vk2,
                                      verbose = verbose, Gauss = TRUE, N = N,
                                      x_tail = x_tail)
   q_wschisq(u = u, weights = weights_dfs$weights, dfs = weights_dfs$dfs,
             ncps = ncps, method = method, ...)
 
 }
+
+
+#' @title Finite Sobolev statistics for testing (hyper)spherical uniformity
+#'
+#' @description Computes the finite Sobolev statistic \deqn{
+#' S_{n, p}(\{b_{k, p}\}_{k=1}^K) = \sum_{i, j = 1}^n
+#' \sum_{k = 1}^K b_{k, p}C_k^(p / 2 - 1)(\cos^{-1}({\bf X}_i'{\bf X}_j)),}
+#' for a sequence  \eqn{\{b_{k, p}\}_{k = 1}^K} of non-negative weights. For
+#' \eqn{p = 2}, the Gegenbauer polynomials are replaced by Chebyshev ones.
+#' @inheritParams sph_stat
+#' @inheritParams cir_stat
+#' @param vk2 weights for the finite Sobolev test. A non-negative vector or
+#' matrix. Defaults to \code{c(0, 0, 1)}.
+#' @return A matrix of size \code{c(M, ncol(vk2))} containing the statistics for
+#' each of the \code{M} samples.
+#' @export
+sph_stat_Sobolev <- function(X, Psi_in_X = FALSE, p = 0, vk2 = c(0, 0, 1)) {
+
+  # Compute Psi matrix with angles between pairs?
+  if (Psi_in_X) {
+
+    n <- n_from_dist_vector(nrow(X))
+    if (p == 0) {
+
+      stop("p >= 2 must be specified if Psi_in_X = TRUE.")
+
+    }
+    X <- X[, , 1, drop = FALSE]
+    dim(X) <- dim(X)[1:2]
+    M <- ncol(X)
+
+  } else {
+
+    n <- nrow(X)
+    p <- ncol(X)
+    M <- dim(X)[3]
+    X <- Psi_mat(data = X)
+
+  }
+
+  # Statistics for k with vk2 != 0
+  vk2 <- rbind(vk2)
+  nonzero_vk2 <- which(apply(vk2 != 0, 2, any))
+  Tnk <- matrix(0, nrow = M, ncol = ncol(vk2))
+  for (j in seq_len(M)) {
+
+      Tnk[j, nonzero_vk2] <- rowSums(Gegen_polyn(theta = X[, j],
+                                                 k = nonzero_vk2, p = p))
+
+  }
+
+  # Get Gegenbauer coefficients
+  bk <- vk2_to_bk(vk2 = vk2, p = p)
+
+  # Construct statistic, a matrix of size c(M, length(bk))
+  Tn <- (2 / n)  * (Tnk %*% t(bk))
+
+  # Add diagonal bias
+  bias <- drop(bk[, nonzero_vk2] %*% Gegen_polyn(theta = 0, k = nonzero_vk2,
+                                                 p = p))
+  Tn <- t(t(Tn) + bias)
+  return(unname(Tn))
+
+}
+
+
+#' @rdname sph_stat_Sobolev
+#' @export
+cir_stat_Sobolev <- function(Theta, Psi_in_Theta = FALSE, vk2 = c(0, 0, 1)) {
+
+  if (Psi_in_Theta) {
+
+    if (length(dim(Theta)) < 3) {
+
+      dim(Theta) <- c(dim(Theta), 1)
+
+    }
+    return(sph_stat_Sobolev(X = Theta, Psi_in_X = TRUE, p = 2, vk2 = vk2))
+
+  } else {
+
+    return(sph_stat_Sobolev(X = Theta_to_X(Theta), Psi_in_X = FALSE, vk2 = vk2))
+
+  }
+
+}
