@@ -13,8 +13,8 @@ arma::uword n_from_dist_vector(arma::uword n_dist);
 arma::vec ecdf_bin(arma::vec data, arma::vec sorted_x, bool data_sorted,
                    bool efic, bool divide_n);
 arma::vec cir_stat_An_Psi(arma::mat Psi, arma::uword n);
-arma::vec cir_stat_Rothman_Psi(arma::mat Psi, double t_m2, double t_min2,
-                               arma::uword n);
+arma::vec cir_stat_Rothman_Psi(arma::mat Psi, arma::uword n, double t_m2,
+                               double t_min2);
 arma::vec cir_stat_Cressie(arma::mat Theta, double t, bool sorted);
 arma::vec sph_stat_Bingham(arma::cube X);
 arma::vec cir_stat_Hermans_Rasson_Psi(arma::mat Psi, arma::uword n);
@@ -25,12 +25,17 @@ arma::vec cir_stat_Pycke_q_Psi(arma::mat Psi, arma::uword n, double q);
 arma::vec sph_stat_Riesz(arma::cube X, bool Psi_in_X, arma::uword p, double s);
 arma::vec sph_stat_PCvM(arma::cube X, bool Psi_in_X, arma::uword p,
                        arma::uword N, arma::uword L);
-arma::vec sph_stat_PRt(arma::cube X, double t,  bool Psi_in_X, arma::uword p,
+arma::vec sph_stat_PRt(arma::cube X, bool Psi_in_X, arma::uword p, double t,
                        arma::uword N, arma::uword L);
 arma::vec sph_stat_PAD(arma::cube X, bool Psi_in_X, arma::uword p,
                        arma::uword N, arma::uword L);
 arma::vec sph_stat_CCF09(arma::cube X, arma::mat dirs, arma::uword K_CCF09,
                          bool original = false);
+arma::vec sph_stat_Poisson(arma::cube X, bool Psi_in_X, arma::uword p,
+                           double rho);
+arma::vec sph_stat_Softmax(arma::cube X, bool Psi_in_X, arma::uword p,
+                           double kappa);
+arma::vec sph_stat_Stereo(arma::cube X, bool Psi_in_X, arma::uword p, double a);
 
 // Constants
 const double pi = M_PI;
@@ -545,8 +550,8 @@ arma::vec cir_stat_An_Psi(arma::mat Psi, arma::uword n) {
 //' @rdname cir_stat
 //' @export
 // [[Rcpp::export]]
-arma::vec cir_stat_Rothman(arma::mat Theta, double t = 1.0 / 3.0,
-                           bool Psi_in_Theta = false) {
+arma::vec cir_stat_Rothman(arma::mat Theta, bool Psi_in_Theta = false,
+                           double t = 1.0 / 3.0) {
 
   // Sample size
   arma::uword n = Psi_in_Theta ?
@@ -564,7 +569,7 @@ arma::vec cir_stat_Rothman(arma::mat Theta, double t = 1.0 / 3.0,
   if (Psi_in_Theta) {
 
     // Compute statistic
-    return cir_stat_Rothman_Psi(Theta, t_m2, t_min2, n);
+    return cir_stat_Rothman_Psi(Theta, n, t_m2, t_min2);
 
   } else {
 
@@ -579,7 +584,7 @@ arma::vec cir_stat_Rothman(arma::mat Theta, double t = 1.0 / 3.0,
       arma::mat Psi = Psi_mat(Theta_cube, ind_tri, true, false, false);
 
       // Compute statistic
-      Ant(k) = arma::as_scalar(cir_stat_Rothman_Psi(Psi, t_m2, t_min2, n));
+      Ant(k) = arma::as_scalar(cir_stat_Rothman_Psi(Psi, n, t_m2, t_min2));
 
     }
 
@@ -592,8 +597,8 @@ arma::vec cir_stat_Rothman(arma::mat Theta, double t = 1.0 / 3.0,
 
 //' @keywords internal
 // [[Rcpp::export]]
-arma::vec cir_stat_Rothman_Psi(arma::mat Psi, double t_m2, double t_min2,
-                               arma::uword n) {
+arma::vec cir_stat_Rothman_Psi(arma::mat Psi, arma::uword n, double t_m2,
+                               double t_min2) {
 
   // Statistic
   arma::vec Ant = -arma::sum(arma::clamp(Psi * inv_two_M_PI - t_m2,
@@ -1061,7 +1066,7 @@ arma::vec cir_stat_Bakshaev(arma::mat Theta, bool Psi_in_Theta = false) {
 //' @rdname cir_stat
 //' @export
 // [[Rcpp::export]]
-arma::vec cir_stat_Riesz(arma::mat Theta, bool Psi_in_Theta = false, 
+arma::vec cir_stat_Riesz(arma::mat Theta, bool Psi_in_Theta = false,
                          double s = 1.0) {
 
   if (Psi_in_Theta) {
@@ -1102,18 +1107,18 @@ arma::vec cir_stat_PCvM(arma::mat Theta, bool Psi_in_Theta = false) {
 //' @rdname cir_stat
 //' @export
 // [[Rcpp::export]]
-arma::vec cir_stat_PRt(arma::mat Theta, double t = 1.0 / 3.0,
-                       bool Psi_in_Theta = false) {
+arma::vec cir_stat_PRt(arma::mat Theta, bool Psi_in_Theta = false,
+                       double t = 1.0 / 3.0) {
 
   if (Psi_in_Theta) {
 
     arma::cube Theta_cube(Theta.n_rows, Theta.n_cols, 1);
     Theta_cube.slice(0) = Theta;
-    return sph_stat_PRt(Theta_cube, t, true, 2, 0, 0);
+    return sph_stat_PRt(Theta_cube, true, 2, t, 0, 0);
 
   } else {
 
-    return sph_stat_PRt(Theta_to_X(Theta), t, Psi_in_Theta, 2, 0, 0);
+    return sph_stat_PRt(Theta_to_X(Theta), Psi_in_Theta, 2, t, 0, 0);
 
   }
 
@@ -1172,6 +1177,48 @@ arma::vec cir_stat_PAD(arma::mat Theta, bool Psi_in_Theta = false,
 }
 
 
+//' @rdname cir_stat
+//' @export
+// [[Rcpp::export]]
+arma::vec cir_stat_Poisson(arma::mat Theta, bool Psi_in_Theta = false,
+                           double rho = 0.5) {
+
+  if (Psi_in_Theta) {
+
+    arma::cube Theta_cube(Theta.n_rows, Theta.n_cols, 1);
+    Theta_cube.slice(0) = Theta;
+    return sph_stat_Poisson(Theta_cube, true, 2, rho);
+
+  } else {
+
+    return sph_stat_Poisson(Theta_to_X(Theta), false, 2, rho);
+
+  }
+
+}
+
+
+//' @rdname cir_stat
+//' @export
+// [[Rcpp::export]]
+arma::vec cir_stat_Softmax(arma::mat Theta, bool Psi_in_Theta = false,
+                           double kappa = 1) {
+
+ if (Psi_in_Theta) {
+
+   arma::cube Theta_cube(Theta.n_rows, Theta.n_cols, 1);
+   Theta_cube.slice(0) = Theta;
+   return sph_stat_Softmax(Theta_cube, true, 2, kappa);
+
+ } else {
+
+   return sph_stat_Softmax(Theta_to_X(Theta), false, 2, kappa);
+
+ }
+
+}
+
+
 /*
  * Other tests
  */
